library DataDump1;

{ Important note about DLL memory management: ShareMem must be the
  first unit in your library's USES clause AND your project's (select
  Project-View Source) USES clause if your DLL exports any procedures or
  functions that pass strings as parameters or function results. This
  applies to all strings passed to and from your DLL--even those that
  are nested in records and classes. ShareMem is the interface unit to
  the BORLNDMM.DLL shared memory manager, which must be deployed along
  with your DLL. To avoid using BORLNDMM.DLL, pass string information
  using PChar or ShortString parameters. }

uses
  SysUtils,
  Classes,
  Types;

const
  DLLVERN = 3;
  DLLVERINTN = 1;
  SIDES = 3;
  SLIVER360 = 134217728;

type
  URegion = packed record  // uncertainty region
    sLon : packed array[0..3] of longint; // in slivers, to deg (double)((sLon*360.0)/SLIVER360))
    sLat : packed array[0..3] of longint; // in slivers, to deg (double)(90-(sLat*360.0/SLIVER360))
  end;
  PURegion = ^URegion;

  TSidesUChar = packed array[0..SIDES-1] of byte;
  PTSidesUChar = ^TSidesUChar; // GE will be passing a pointer so we need to receive as one

  TURegion = packed array[0..SIDES-1] of PURegion;
  PTURegion = ^TURegion;

var
  txtDataDump: System.TextFile;


{*** Return DLL internal version, i.e. revision 5 of my DLL ***}
function DLLVer():word;export;cdecl;
begin
  result := DLLVERN;
end;

{*** Return DLL interface version, i.e. revision 1 of my HCE interface spec ***}
function DLLVerInt():word;export;cdecl;
begin
  result := DLLVERINTN;
end;

{*** Return DLL description, 255 characters max ***}
function DLLDesc():pchar;export;cdecl;
begin
  result := 'Data Dump to Text DLL by Tony Eischens';
end;

{*** Free DLL ***}
procedure DLLFree();export;cdecl;
begin
  // Do any cleanup here, this is the end
  Flush(txtDataDump);
  CloseFile(txtDataDump);
end;

{*** Init DLL ***}
procedure DLLInit();export;cdecl;
begin
  // here we initialize globals and such for the life of the DLL
  AssignFile(txtDataDump, 'datadump.txt');
  SetLineBreakStyle(txtDataDump, tlbsCRLF);
//  if FileExists('datadump.txt') then
//    Append(txtDataDump) // append
//  else
    Rewrite(txtDataDump); // overwrite
end;

{*** receive lat/lon info ***}
procedure UnitLatLon(uID: byte; gID: pchar; sLat: Double; sLon: Double);export;cdecl;
begin
  // Dump data to text file
  Writeln(txtDataDump, Format('%0.3d, %s, UnitLatLon, %8.8f, %8.8f', [uID, gID, sLat, sLon]));
end;

{*** receive visibility info ***}
procedure UnitShowing(uID: byte; gID: pchar; showing: PTSidesUChar);export;cdecl;
var
  intC: integer;
  strText: string;
begin
  strText := Format('%0.3d, %s, UnitShowing, ', [uID, gID]);
  for intC := 0 to SIDES-1 do
  begin
    // Dump data to text file
    strText := strText + Format('%0.3d', [showing[intC]]);
  end;
//  strText := strText + IntToStr(showing.side0);
  Writeln(txtDataDump, strText);
end;

{*** receive uncertainty region info ***}
procedure UnitURegion(uID: byte; gID: pchar; URegs: PTURegion);export;cdecl;
var
  intC1, intC2: integer;
  strText: string;
begin
  strText := Format('%0.3d, %s, UnitURegion', [uID, gID]);
  for intC1 := 0 to SIDES-1 do
  begin
    for intC2 := 0 to 3 do
    begin
      // Dump data to text file
      // (double)((sLon*360.0)/SLIVER360))
      // (double)(90-(sLat*360.0/SLIVER360))
      strText := strText + Format(', %8.8f, %8.8f',
                                   [(90-(URegs[intC1].sLat[intC2]*360.0/SLIVER360)),
                                   ((URegs[intC1].sLon[intC2]*360.0)/SLIVER360)]);
    end;
  end;
  Writeln(txtDataDump, strText);
end;

{*** receive course info ***}
procedure UnitCourse(uID: byte; gID: pchar; course: smallint);export;cdecl;
begin
  Writeln(txtDataDump, Format('%0.3d, %s, UnitCourse, %0.3d', [uID, gID, course]));
end;

{*** receive speed (kts) info ***}
procedure UnitSpeed(uID: byte; gID: pchar; speed: smallint);export;cdecl;
begin
  Writeln(txtDataDump, Format('%0.3d, %s, UnitSpeed, %0.3d', [uID, gID, speed]));
end;

{*** receive altitude band info ***}
procedure UnitAlt(uID: byte; gID: pchar; alt: byte);export;cdecl;
begin
  Writeln(txtDataDump, Format('%0.3d, %s, UnitAlt, %0.2d', [uID, gID, alt]));
end;

{*** receive unit 'color', detected side info ***}
procedure UnitShowingSide(uID: byte; gID: pchar; showing: PTSidesUChar);export;cdecl;
var
  intC: integer;
  strText: string;
begin
  strText := Format('%0.3d, %s, UnitShowingSide, ', [uID, gID]);
  for intC := 0 to SIDES-1 do
  begin
    // Dump data to text file
    strText := strText + Format('%3.3d', [showing[intC]]);
  end;
  Writeln(txtDataDump, strText);
end;

{*** receive unit sink info ***}
procedure UnitSink(uID: byte; gID: pchar);export;cdecl;
begin
  Writeln(txtDataDump, Format('%0.3d, %s, UnitSink', [uID, gID]));
end;

{*** receive gametime ***}
procedure GameTime(GTime: longint);export;cdecl;
begin
  Writeln(txtDataDump, Format('%0.3d, %s, GameTime, %d', [0, '0', GTime]));
end;

{*** receive MessageLog line ***}
procedure MsgLog(MsgLine: pchar);export;cdecl;
begin
  Writeln(txtDataDump, Format('%0.3d, %s, MsgLog, %s', [0, '0', MsgLine]));
end;

{*** receive Unit assignment to group line ***}
procedure UnitToGroup(uID: byte; gID, gIDold: pchar);export;cdecl;
begin
  Writeln(txtDataDump, Format('%0.3d, %s, UnitToGroup, %s', [uID, gID, gIDold]));
end;

{*** receive new unit info ***}
procedure UnitNew(uID: byte; gID: pchar; utype: shortint; side: byte; annexID: word; name: pchar);export;cdecl;
begin
  Writeln(txtDataDump, Format('%0.3d, %s, UnitNew, %d, %d, %0.5d, %s', [uID, gID, utype, side, annexID, name]));
end;

{*** receive new unit free info ***}
procedure UnitFree(uID: byte; gID: pchar);export;cdecl;
begin
  Writeln(txtDataDump, Format('%0.3d, %s, UnitFree', [uID, gID]));
end;

{*** receive new scenario/save info ***}
procedure ScenLoad(strHarpPath: pchar; intScenNum: SmallInt; strScenFPath: pchar; intBSNum: ShortInt; strBSFName: pchar);export;cdecl;
begin
  Writeln(txtDataDump, Format('%0.3d, %s, ScenLoad, %s, %d, %s, %d, %s', [0, '0', strHarpPath, intScenNum, strScenFPath, intBSNum, strBSFName]));
end;

exports
  DLLVer,
  DLLVerInt,
  DLLDesc,
  DLLFree,
  DLLInit,
  UnitLatLon,
  UnitShowing,
  UnitURegion,
  UnitCourse,
  UnitSpeed,
  UnitAlt,
  UnitShowingSide,
  UnitSink,
  GameTime,
  MsgLog,
  UnitToGroup,
  UnitNew,
  UnitFree,
  ScenLoad;
end.

